//
// Crater.cpp
// A crater on the moon.
//
//    Copyright 2002 by Akkana Peck.
// You may use or distribute this code under the terms of the GPL.
//

#include "Crater.h"

#include <qpainter.h>
#include <math.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>
#include <stdio.h>

// Static variables:
int Crater::sMoonRadius = 1;
int Crater::sCurIndex = 0;
Crater* Crater::sCraterList = 0;
int Crater::sNumCraters = 0;

Crater::Crater() : mName(0)
{
}

Crater::Crater(char* name, char* desc, int size,
               int longitude, int latitude)
{
    Init(name, desc, size, longitude, latitude);
}

void
Crater::Init(char* name, char* desc, int size,
             int longitude, int latitude)
{
    mName = strdup(name);
    if (desc)
        mDesc = strdup(desc);
    else
        mDesc = 0;
    mSize = size;
    mLongitude = longitude;
    mLatitude = latitude;
    if (sMoonRadius >= 1)
    {
        double longRadians = M_PI / 180. * mLongitude;
        double latRadians = M_PI / 180. * mLatitude;
        double cosLatRadians = cos(latRadians);
        mX = (int)(sMoonRadius * sin(longRadians) * cosLatRadians);
        mY = (int)(-sMoonRadius * sin(latRadians));
        double size = (double)mSize * sMoonRadius / 1609.;
        mSizeX = (int)(size * cos(longRadians));
        mSizeY = (int)(size * cosLatRadians);
    }
}

Crater::~Crater()
{
    if (mName) delete[] mName;
    if (mDesc) delete[] mDesc;
}

// Find the distance from a point to the crater's center.
// If the point is inside the crater, we also set isInside.
double Crater::Distance(double longitude, double latitude, int* isInside)
{
    double dLat = latitude - mLatitude;
    double dLong = longitude - mLongitude;
    double thisdist = sqrt(dLat*dLat + dLong*dLong) * 180. / sMoonRadius;
    if (isInside)
        *isInside = (thisdist < mSizeX/2 && thisdist < mSizeY/2);
    return thisdist;
}

void
Crater::Paint(QPainter* p, int xcenter, int ycenter, int moonRadius)
{
    // This crashes.  I don't know why.
    int x = (int)(xcenter + moonRadius * sin(M_PI/180.*mLongitude));
    int y = (int)(ycenter + moonRadius * sin(M_PI/180.*mLatitude));
    int size = (int)(mSize / 16.72);
    int size2 = size/2;
            
    p->drawEllipse(x-size2, y-size2, size, size);
}

// Routines for looping over all known craters:
void
Crater::ReadCraterFile(char* craterFileName, int moonRadius)
{
    sMoonRadius = moonRadius;

    // Delete the old crater list:
    if (sCraterList)
        delete[] sCraterList;

    sNumCraters = 0;

    // XXX for now, don't read the file, just init with two objects:
    sCraterList = (Crater*)malloc((sizeof (Crater)) * 2000);
    if (!sCraterList)
        return;

    FILE* cfile = fopen(craterFileName, "r");
    if (!cfile)
        return;

    char line[BUFSIZ];
    while (fgets(line, sizeof line, cfile) != 0)
    {
        int latitude = (int)atof(line+30);
        if (line[36] == 'S')
            latitude = -latitude;
        int longitude = (int)atof(line+38);
        if (line[44] == 'W')
            longitude = -longitude;
        // Write a null after the name in the line buffer,
        // so we can pass it to strdup:
        int i;
        for (i=28; i>0 && line[i] == ' '; --i)
            ;
        if (i <= 0)
            return;

        line[i+1] = '\0';

        sCraterList[sNumCraters++].Init(line, line+56,
                                        (int)atof(line+47),
                                        longitude, latitude);
    }
    printf("%d craters loaded\n", sNumCraters);
}

void Crater::InitLoop()
{
    sCurIndex = -1;
}

Crater* Crater::NextCrater()
{
    if (++sCurIndex >= sNumCraters)
        return 0;
    return &sCraterList[sCurIndex];
}

